declare type WritableValue = string | number | boolean | WritableValue[] | {
    [key: string]: WritableValue;
};
export interface AnyState {
    [key: string]: WritableValue;
}
export declare type Serializer<S extends AnyState> = (state: S) => string;
export declare type Deserializer<S extends AnyState> = (state: string) => S;
export declare type Setter<S extends AnyState> = <K extends keyof S>(state: S, key: K, value: S[K]) => S;
export declare type Getter<S extends AnyState> = <K extends keyof S>(state: S, key: K) => S[K];
export declare type Equality<S extends AnyState> = (state: S, newState: S) => boolean;
export interface DotPrefOptions<S extends AnyState> {
    defaults: S;
    name: string;
    filename: string;
    dirPath: string;
    serializer: Serializer<S>;
    deserializer: Deserializer<S>;
    encoder: (decodedString: string) => string;
    decoder: (encodedString: string) => string;
    setter: Setter<S>;
    getter: Getter<S>;
    equality: Equality<S> | boolean;
}
export interface DotPref<S extends AnyState> {
    get: <K extends keyof S>(key: K) => S[K];
    set: <K extends keyof S>(key: K, value: S[K]) => void;
    reset: <K extends keyof S>(key: K) => void;
    read: () => void;
    write: () => void;
    filePath: string;
}
export {};
