"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.getDefaultCrypto = exports.getKey = exports.SSH_KEY_PATH = exports.OUTPUT_ENCODING = exports.INPUT_ENCODING = void 0;
const crypto_1 = require("crypto");
const fs_1 = require("fs");
const os_1 = require("os");
const path_1 = require("path");
exports.INPUT_ENCODING = 'utf8';
exports.OUTPUT_ENCODING = 'hex';
exports.SSH_KEY_PATH = (0, path_1.join)((0, os_1.homedir)(), '.ssh', 'id_rsa');
const readKey = (keyPath) => (0, fs_1.existsSync)(keyPath) && (0, fs_1.readFileSync)(keyPath).toString(exports.INPUT_ENCODING);
const getKey = ({ keyPath, defaultKey = 'PREF' } = {}) => readKey(keyPath) || readKey(exports.SSH_KEY_PATH) || defaultKey;
exports.getKey = getKey;
const getDefaultCrypto = (config = {}) => {
    const key = (0, exports.getKey)(config);
    const encryptedKey = (0, crypto_1.createHash)('sha256').update(key).digest();
    const encoder = (text) => {
        const iv = (0, crypto_1.randomBytes)(8).toString(exports.OUTPUT_ENCODING);
        const cipher = (0, crypto_1.createCipheriv)('aes256', encryptedKey, iv);
        const encrypted = cipher.update(text, exports.INPUT_ENCODING, exports.OUTPUT_ENCODING) +
            cipher.final(exports.OUTPUT_ENCODING);
        return `${iv.toString()}:${encrypted.toString()}`;
    };
    const decoder = (text) => {
        const [ivString, encrypted] = text.split(':');
        const decipher = (0, crypto_1.createDecipheriv)('aes256', encryptedKey, ivString);
        return (decipher.update(encrypted, exports.OUTPUT_ENCODING, exports.INPUT_ENCODING) +
            decipher.final(exports.INPUT_ENCODING));
    };
    return {
        encoder,
        decoder,
    };
};
exports.getDefaultCrypto = getDefaultCrypto;
